/* Cyber Kicker, a utility to remotely reset a develoment board
 *
 * Written by Matt Mets in 2007
 * This code is released into the public domain
 *
 * Adapted from an example by Tomi Engdahl <then@delta.hut.fi>
 * http://www.epanorama.net/circuits/parallel_output.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/io.h>

#define base 0x378      /* base address of parallel port */

#define PULSE_PIN_OK            0
#define PULSE_PIN_BAD_PIN       1
#define PULSE_PIN_BAD_TIME      2
#define PULSE_PIN_PORT_ERROR    3

/* Pulse the selected pin high for a given amount of time
 *
 * address: address of the parallel port
 * pin_number: pin 0-7 (corresponds to pin 2-9 on the parallel port)
 * time: time to pulse the pin for, in seconds
 */
int pulsePin( int address, int pin_number , int time ) {
    unsigned char pin_mask;

    if( (pin_number < 1) || (pin_number > 8) )
        return PULSE_PIN_BAD_PIN;

    if( (time < 1) || (time > 10) )
        return PULSE_PIN_BAD_TIME;

    if( ioperm( address,1,1 ) )
        return PULSE_PIN_PORT_ERROR;
    
    printf( "Pulsing Pin #%i for %i seconds:\n", pin_number, time );

    pin_mask = 1 << (pin_number - 1);

    outb( pin_mask, address );
    sleep( time );
    outb( 0, address );

    return PULSE_PIN_OK;
}


int main( int argc, char **argv ) {
    int ret;

    if( argc != 3 ) {
        printf( "%s: incorrect number of arguments specified\n", argv[0] );
        printf( "Usage: %s pin# time\n", argv[0] );
        printf( "Options:\n" );
        printf( "  pin#: Pin to be toggled, in the range 1 to 8\n" );
        printf( "  time: Length of time to toggle pin for, in seconds.\n" );
        printf( "        Valid times are 1 to 10 seconds\n" );
        return PULSE_PIN_BAD_PIN;
    }

    ret = pulsePin( base, atoi(argv[1]), atoi(argv[2]) );
    switch( ret ) {
        case PULSE_PIN_OK:
            printf( "Success\n" );
            break;
        case PULSE_PIN_BAD_PIN:
            printf( "%s: Pin value out of range (must be 1-8)\n", argv[0] );
            break;
        case PULSE_PIN_BAD_TIME:
            printf( "%s: Time value out of range (must be 1-10)\n", argv[0]);
            break;
        case PULSE_PIN_PORT_ERROR:
            printf( "%s: Unable to open port for I/O (suid?)\n", argv[0] );
            break;
    }
    return ret;
}
